function getParameterByName(name)
{
  name = name.replace(/[\[]/, "\\[").replace(/[\]]/, "\\]");
  var regexS = "[\\?&]" + name + "=([^&#]*)";
  var regex = new RegExp(regexS);
  var results = regex.exec(window.location.search);
  if(results == null)
    return "";
  else
    return decodeURIComponent(results[1].replace(/\+/g, " "));
}

/**
 * Removes unnecessary query string parts from callback url
 * @returns callbackURL as string
 */
function prepareCallbackURL()
{
	var queryStringPos = document.location.href.indexOf('?');
	var callbackURL = document.location.href;
	if (-1 != queryStringPos) {
		callbackURL = document.location.href.substring(0, queryStringPos);
	}
	if ('expert' === getParameterByName('mode')) {
		callbackURL =  callbackURL + '?mode=expert';
	}
	return callbackURL;
}


function getStateGridTextActive (scene)
{
    var scene_name = dss.staticDataModel.getNameForScene(0, 0, scene);
    var gridTextActive = Ext.String.format2(_("only when {name} is active"), {'name': scene_name});
    return gridTextActive;
}

function getStateGridTextInactive (scene)
{
    var scene_name = dss.staticDataModel.getNameForScene(0, 0, scene);
    var gridTextInactive = Ext.String.format2(_("only when {name} is inactive"), {'name': scene_name});
    return gridTextInactive;
}

var waitWindow;
var messageBox;
var oEventTwitterer;
var userAccount = "";
var oGrid=null;

Ext.define('DSS.addon.EventTwitterer', {

    extend : 'DSS.addon.Framework',

    config : {
        appName : 'Event-Twitterer',
        appId : 'event-twitterer',
        appVersion : '1.0.6',
        dSSVersion : '1.9.0',
        appIcon : 'default_icon.png'
    },

    getHelp : function() {
        return Ext.create('Ext.Component', {
            whiteSpace : 'normal',
            padding : '0 20 0 0', //top right bottom left
            loader : {
                url : 'locale/' + dss.staticDataModel.activeLanguage + '/help.html',
                autoLoad : true,
                disableCaching : false
            }
        });
    },

    getContent : function() {
        var me=this;
        
        var conditions = [{type:'singleItem',title:_('state_daynight'),entry:[
                              {type:'state',name:'daylight',value:true,title:Ext.String.format2(_('daynight_bright'),{
                                        duskTime:dss.staticDataModel.getDusk(),
                                        dawnTime:dss.staticDataModel.getDawn()
                                    }),gridText:Ext.String.format2(_('grid_daynight_twilight_true'),{
                                        duskTime:dss.staticDataModel.getDusk(),
                                        dawnTime:dss.staticDataModel.getDawn()
                                    })},
                              {type:'state',name:'daylight',value:false,title:Ext.String.format2(_('daynight_dark'),{
                                    duskTime:dss.staticDataModel.getDusk(),
                                    dawnTime:dss.staticDataModel.getDawn()
                                }),gridText:Ext.String.format2(_('grid_daynight_twilight_false'),{
                                    duskTime:dss.staticDataModel.getDusk(),
                                    dawnTime:dss.staticDataModel.getDawn()
                                })},
                              {type:'state',name:'daynight',value:true,title:Ext.String.format2(_('daynight_day'),{
                                    sunriseTime:dss.staticDataModel.getSunrise(),
                                    sunsetTime:dss.staticDataModel.getSunset()
                                }),gridText:Ext.String.format2(_('grid_daynight_daynight_true'),{
                                    sunriseTime:dss.staticDataModel.getSunrise(),
                                    sunsetTime:dss.staticDataModel.getSunset()
                                })},
                              {type:'state',name:'daynight',value:false,title:Ext.String.format2(_('daynight_night'),{
                                    sunriseTime:dss.staticDataModel.getSunrise(),
                                    sunsetTime:dss.staticDataModel.getSunset()
                                }),gridText:Ext.String.format2(_('grid_daynight_daynight_false'),{
                                    sunriseTime:dss.staticDataModel.getSunrise(),
                                    sunsetTime:dss.staticDataModel.getSunset()
                                })}]},
                          {type:'timeframe',title:_('Timeframe')},
                          {type:'weekdays',title:_('Weekdays')},
                          {type:'dateframe',title:_('Dateframe')},
                          {type:'deviceState',title:_('automatic devices')},
                          {type:'singleItem',title:_('state_presence'),entry: [
                              {type:'state',name:'presence',value:'present',title:_('presence_present'),gridText:_('grid_presence_present')},
                              {type:'state',name:'presence',value:'absent',title:_('presence_absent'),gridText:_('grid_presence_absent')}]},
                          {type:'singleItem',title:_('state_holiday'),entry: [
                              {type:'state',name:'holiday',value:'on',title:_('holiday_on'),gridText:_('grid_holiday_on')},
                              {type:'state',name:'holiday',value:'off',title:_('holiday_off'),gridText:_('grid_holiday_off')}]},
                          {type:'multiItem',title:_('weather trigger'),entry: [
                              {type:'state',name:'rain',value:'1',title:_('rain active'),gridText:_('grid_rain_active'),group:1},
                              {type:'state',name:'rain',value:'2',title:_('rain inactive'),gridText:_('grid_rain_inactive'),group:1},
                              {type:'state',name:'wind',value:'1',title:_('wind active'),gridText:_('grid_wind_active'),group:2},
                              {type:'state',name:'wind',value:'2',title:_('wind inactive'),gridText:_('grid_wind_inactive'),group:2},
                              {type:'state',name:'hail',value:'1',title:_('hail active'),gridText:_('grid_hail_active'),group:3},
                              {type:'state',name:'hail',value:'2',title:_('hail inactive'),gridText:_('grid_hail_inactive'),group:3},
                              {type:'state',name:'frost',value:'1',title:_('frost active'),gridText:_('grid_frost_active'),group:4},
                              {type:'state',name:'frost',value:'2',title:_('frost inactive'),gridText:_('grid_frost_inactive'),group:4},
                              {type:'state',name:'sun',value:'1',title:_('sun active'),gridText:_('grid_sun_active'),group:5},
                              {type:'state',name:'sun',value:'2',title:_('sun inactive'),gridText:_('grid_sun_inactive'),group:5}]},
                              {type:'state',name:'service',value:'1',title:_('service active'),gridText:_('grid_service_active'),group:6},
                              {type:'state',name:'service',value:'2',title:_('service inactive'),gridText:_('grid_service_inactive'),group:6},
                         {type:'multiItem',title:_('Security'),entry: [
                              {type:'state',name:'fire',value:'1',title:_('fire active'),gridText:_('grid_fire_active'),group:1},
                              {type:'state',name:'fire',value:'2',title:_('fire inactive'),gridText:_('grid_fire_inactive'),group:1},
                              {type:'state',name:'panic',value:'1',title:_('panic_active'),gridText:_('grid_panic_active'),group:2},
                              {type:'state',name:'panic',value:'2',title:_('panic_inactive'),gridText:_('grid_panic_inactive'),group:2},
                              {type:'state',name:'alarm',value:'1',title:getStateGridTextActive(74),gridText:getStateGridTextActive(74), group:3},
                              {type:'state',name:'alarm',value:'2',title:getStateGridTextInactive(74),gridText:getStateGridTextInactive(74),group:3},
                              {type:'state',name:'alarm2',value:'1',title:getStateGridTextActive(83),gridText:getStateGridTextActive(83),group:4},
                              {type:'state',name:'alarm2',value:'2',title:getStateGridTextInactive(83),gridText:getStateGridTextInactive(83),group:4},
                              {type:'state',name:'alarm3',value:'1',title:getStateGridTextActive(84),gridText:getStateGridTextActive (84),group:5},
                              {type:'state',name:'alarm3',value:'2',title:getStateGridTextInactive(84),gridText:getStateGridTextInactive(84),group:5},
                              {type:'state',name:'alarm4',value:'1',title:getStateGridTextActive(85),gridText:getStateGridTextActive(85),group:6},
                              {type:'state',name:'alarm4',value:'2',title:getStateGridTextInactive(85),gridText:getStateGridTextInactive(85),group:6}]},
                          {type:'zonestate',title:_('ZoneState')},
                          {type:'clusterstate',title:_('Cluster State')}];

        var climateSettings = dss.staticDataModel.getClimateSettings();
        if (climateSettings != null && climateSettings.capabilities != null && climateSettings.capabilities.coolingSupported == true) {
            conditions.push({type:'multiItem',title:_('Climate system'),entry: [
                                {type:'state',name:'heating_water_system',value:'0',title:_('off'),gridText:_('grid_climate_off'),group:1},
                                {type:'state',name:'heating_water_system',value:'1',title:_('heating on'),gridText:_('grid_climate_heating'),group:1},
                                {type:'state',name:'heating_water_system',value:'2',title:_('cooling on'),gridText:_('grid_climate_cooling'),group:1}]
            });
        } else {
            conditions.push({type:'multiItem',title:_('Climate system'),entry: [
                                {type:'state',name:'heating_water_system',value:'0',title:_('off'),gridText:_('grid_climate_off'),group:1},
                                {type:'state',name:'heating_water_system',value:'1',title:_('heating on'),gridText:_('grid_climate_heating'),group:1}]
            });
        }
       
        var oConfig={
                defaultSortCol:'name',
                disabledProperty:'disabled',
                storeConfig: {
                    storeId : 'emailStore',
                    fields : [ {name:'disabled', sortType:'asBoolean'}, 
                               {name:'name', sortType:'asUCText'}, 
                               {name:'id'}, 
                               {name:'message', sortType:'asUCText'}, 
                               {name:'triggers', sortType:'asTrigger'}, 
                               {name:'conditions',  sortType: 'asUCText' },
                               {name:'grouping',  sortType: 'asUCText' },
                               {name:'defaultFooter', sortType:'asBoolean'} ]
                },
                multiselect:true,
                columns: [{
                    header : _('Active'),
                    dataIndex : 'disabled',
                    resizable : false,
                    xtype: 'disabledcolumn', 
                    width : 50
                }, {
                    header : _('Name'),
                    dataIndex : 'name',
                    xtype: 'textcolumn', 
                    width : 200
                }, {
                    header : _('Trigger'),
                    dataIndex : 'triggers', 
                    maintainFlex : true,
                    flex : 1,
                    xtype: 'triggercolumn'
                },{
                    header : _('Message'),
                    dataIndex : 'message',
                    xtype:'textcolumn',
                    flex : 1
                },{
                    header: _('Conditions'),
                    dataIndex: 'conditions', // alibi index
                    xtype: 'conditioncolumn',
                    maintainFlex: true,
                    flex: 1,
                    customRendererOptions:{hideEnabled:true,hideWeekdays:false,hideGenericSystemStates:false,hideZoneStates:false,hideTimeframe:false,filterSystemState:[],
                        stateTitles:[{name:'daylight',value:true,title:_('grid_daylight_true')},{name:'daylight',value:false,title:_('grid_daylight_false')},
                                     {name:'daynight',value:true,title:_('grid_daynight_true')},{name:'daynight',value:false,title:_('grid_daynight_false')},
                                     {name:'panic',value:'1',title:_('grid_panic_active')},{name:'panic',value:'2',title:_('grid_panic_inactive')},
                                     {name:'presence',value:'present',title:_('grid_presence_present')},{name:'presence',value:'absent',title:_('grid_presence_absent')},
                                     {name:'fire',value:'1',title:_('grid_fire_active')},{name:'fire',value:'2',title:_('grid_fire_inactive')},
                                     {name:'hail',value:'1',title:_('grid_hail_active')},{name:'hail',value:'2',title:_('grid_hail_inactive')},
                                     {name:'wind',value:'1',title:_('grid_wind_active')},{name:'wind',value:'2',title:_('grid_wind_inactive')},
                                     {name:'rain',value:'1',title:_('grid_rain_active')},{name:'rain',value:'2',title:_('grid_rain_inactive')},
                                     {name:'frost',value:'1',title:_('grid_frost_active')},{name:'frost',value:'2',title:_('grid_frost_inactive')},
                                     {name:'alarm',value:'1',title:getStateGridTextActive(74)},{name:'alarm', value:'2',title:getStateGridTextInactive(74)},
                                     {name:'alarm2',value:'1',title:getStateGridTextActive(83)},{name:'alarm2',value:'2',title:getStateGridTextInactive(83)},
                                     {name:'alarm3',value:'1',title:getStateGridTextActive(84)},{name:'alarm3',value:'2',title:getStateGridTextInactive(84)},
                                     {name:'alarm4',value:'1',title:getStateGridTextActive(85)},{name:'alarm4',value:'2',title:getStateGridTextInactive(85)},
                                     {name:'holiday', value:'on',title:_('grid_holiday_on')},{name:'holiday', value:'off',title:_('grid_holiday_off')},
                                     {name:'heating_water_system', value:'0',title:_('grid_climate_off')},{name:'heating_water_system', value:'1',title:_('grid_climate_heating')},{name:'heating_water_system', value:'2',title:_('grid_climate_cooling')}]}
                }],     
                buttonText: {
                    newButton: _('New message'),
                    editButton: _('Edit message'),
                    enableButton:_('Activate message'),
                    disableButton:_('Deactivate message'),
                    testButton: _('Test message'),
                    deleteButton: _('Delete message')
                },
                customButtonsRAlign:[{
                    text:_('Connect to Twitter'),
                    iconCls:'icon-twitter-action',
                    handler:function(grid,entry){
                    	oEventTwitterer.showSignInRequest(grid);
                       
                    }
                },
                {
                    text:_('Disconnect from Twitter'),
                    iconCls:'icon-twitter-action',
                    handler:function(grid,entry){
                    	oEventTwitterer.showAreYouSure(grid);
                    }
                }],

                editorConfig:{
                    title : _('Messageproperties'),
                    staticControls : [{
                        type : 'text',
                        label : _('Name'),
                        id : 'name'
                    }, {
                        type : 'hidden',
                        label : _('id'),
                        id : 'id'
                    }],
                    tabControls :  [{
                        tabLabel : _('Trigger'),
                        tabDescription : _('Choose the triggering activity for the message. You can choose more than one activity as trigger.'),
                        content : [ {
                            type : 'TriggerConfigurator',
                            id : 'triggers',
                            flex : true
                        } ]
                    } , {
                        tabLabel : _('Message'),
                        tabDescription : _('Provide the Message, which will be send.'),
                        content : [ {
                            type : 'TitterMsgControl',
                            id : 'message',
                            title : _('Message'),
                            flex : true
                        }]
                    } , {
                        tabLabel : _('Conditions'),
                        tabDescription : _('Choose the conditions.'),
                        content : [{
                            type : 'ConditionConfigurator',
                            id : 'conditions',
                            conditions:conditions,
                            flex : true
                        }]
                    }]
                },
                listeners: {
                    'loadDataStore':function (grid){
                        me.loadParameter(grid);
                    },
                    'saveEntry':function (grid,values){    
                        
                        me.waitAndReload(grid,{
                            actions : 'save',
                            value : Ext.JSON.encode(values)
                        });
                    },
                    'enableEntry':function (grid,entry){
                    	var idVal;
                    	if (entry.get!=null) {
                    		idVal=entry.get('id');
                    	} else {
                    		idVal=[];
                    		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                    			if (entry[iIndex]!=null)
                    				idVal.push(entry[iIndex].get('id'));
                    	}
                        me.waitAndReload(grid,{
                            actions : 'enable',
                            id : Ext.JSON.encode(idVal)
                        });
                     },
                    'disableEntry':function (grid,entry){
                    	var idVal;
                    	if (entry.get!=null) {
                    		idVal=entry.get('id');
                    	} else {
                    		idVal=[];
                    		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                    			if (entry[iIndex]!=null)
                    				idVal.push(entry[iIndex].get('id'));
                    	}
                        me.waitAndReload(grid,{
                            actions : 'disable',
                            id : Ext.JSON.encode(idVal)
                        });
                    },
                    'deleteEntry':function (grid,entry){
                    	var idVal;
                    	if (entry.get!=null) {
                    		idVal=entry.get('id');
                    	} else {
                    		idVal=[];
                    		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                    			if (entry[iIndex]!=null)
                    				idVal.push(entry[iIndex].get('id'));
                    	}
                        me.waitAndReload(grid,{
                            actions : 'delete',
                            id : Ext.JSON.encode(idVal)
                        });
                    },
                    'testEntry':function (grid,entry){
                        dss.raiseEvent('event-twitterer', {
                            actions : 'test',
                            id : entry.get('id')
                        });
                        startTestResultListener();
                    }
                }
        };
        
        if (getParameterByName('mode')=='expert') {
        	oConfig.grouperField='grouping';
        	oConfig.editorConfig.tabControls[0].content[0].userStates=true;
        }
        
        grid=Ext.create('DSS.gridPanel', oConfig);
        oGrid = grid;
        return grid;
    },
    loadParameter : function(grid) {
    	    	
    	grid.setStatus(_('Loading ...'));
    	
        var sString = dss.ajaxSyncRequest('/json/property/query', {
            'query' : '/scripts/event-twitterer(requestToken,accessToken,userId)'
        });
        var result = Ext.JSON.decode(sString);
        if (result.result!=null) {
        	if (result.result['event-twitterer']!=null) {
        		if (result.result['event-twitterer'][0]!=null) {
        			if (result.result['event-twitterer'][0]['accessToken']==null) {
        				// no final approve
        				if (result.result['event-twitterer'][0]['requestToken'] ==null) {
        					// not signed in
        					this.showSignInRequest(grid);
        				} else {
        					// sign in in progress
        			    	var oauth_token = getParameterByName('oauth_token');
        			    	var oauth_verifier = getParameterByName('oauth_verifier');
        					var bSuccess = verifyTokenMatch(oauth_token, oauth_verifier, result.result['event-twitterer'][0]['requestToken']);
        					if (true === bSuccess) {        							        					
	        					this.showWaitForCompleteSignIn(grid, oauth_token, oauth_verifier);
        					} else {
        						this.showAuthenticationFailed(grid);
        					}
        				}
        			} else {
        				if (result.result['event-twitterer'][0]['userId'] == null) {
        					this.showCredentialsOutdated(grid);
        				}
        			}
        			
        		}
        	}
        }
        sString = dss.ajaxSyncRequest('/json/property/query', {
            'query' : '/scripts/event-twitterer/values/*(*)/*(*)/*(*)/*(*)/*(*)'
        });
        result = Ext.JSON.decode(sString);
        
        oResultObject=dss.parsePropertyQueryAnswer(result,[{name:'name',type:'simple'},
                                                           {name:'id',type:'simple'},
                                                           {name:'disabled',type:'condition/enabled'},
                                                           {name:'message', type:'simple'},
                                                           {name:'triggers',type:'trigger'},
                                                           {name: 'conditions', type: 'condition'}]);    
        if (oResultObject!=null) {
            for (var iIndex=0;iIndex<oResultObject.length;iIndex++) {
                oResultObject[iIndex].grouping=unescape(oResultObject[iIndex].name).split(' ')[0];
            }
            grid.loadParameter(oResultObject);
        }
        this.updateStatus(grid);
    },

    /**
     *  Show confirmation dialog to start 1. step of authentication process
     * @param [in] grid to disable
     */
    showSignInRequest : function (grid) {
		var dialogText = '<ol style="list-style: circle outside none; padding-left: 10px; padding-right: 10px; font: 12px arial, tahoma, verdana, helvetica; color: #000;">'+
		 '<li style="list-style: decimal; margin-left: 20px; padding-bottom: 4px;">' + _('Click on  &lsquo;<i>Connect to Twitter</i>&lsquo; to start the authentication process.') +'</li>' +
		 '<li style="list-style: decimal; margin-left: 20px; padding-bottom: 4px;">' + _('You&lsquo;ll be redirected to a Twitter web site asking you to log in to your account.') +'</li>' +
		 '<li style="list-style: decimal; margin-left: 20px; padding-bottom: 4px;">' + _('After logging in, Twitter will ask you to approve the application.') +'</li>' +
		 '<li style="list-style: decimal; margin-left: 20px; padding-bottom: 4px;">' + _('You will be redirected back to the digitalSTROM applikation where you can start setting up your event notifications.') +'</li></ol>';

    	waitWindow=Ext.create('Ext.window.Window',{
			width:320,
			layout : 'fit',
			padding:"10 10 10 10",
    		header:false,
    		closeable:false,		
    		resizable:false,
 
    		html:'<table style="height:90px;"><tr><td>' + dialogText + 
	     	'</td></tr><tr style="height:100%;"><td style="text-align: center"><button onclick="step1();">' +
	     	'<img height="16" src="images/twitter-bird-light-bgs-small.png">' + _("Connect to Twitter") + '</button>&nbsp; &nbsp; &nbsp;<button onclick="closeSignInDialog(grid);"><img height="16" width="0" src="images/twitter-bird-light-bgs-small.png" style="visibility:hidden">' + _("cancel") + '</button></td>' + 
	     	'</tr></table>'   		    		     
    		
    	});
    	waitWindow.show();
    	grid.disable();    	
    },
    /**
     * Show final wait dialog during step 3 of authetication process
     * @param grid
     * @param oauth_token [in] token received in step 2
     * @param oauth_verifier [in] verifier received in step 2
     */
    showWaitForCompleteSignIn : function(grid, oauth_token, oauth_verifier) {
    	waitWindow=Ext.create('Ext.window.Window',{
    		height:120,
    		width:320,
    		padding:"10 10 10 10",
    		header:false,
    		closeable:false,
    		html:'<table style="height:90px;"><tr><td><p style="text-align: center; font: 12px arial, tahoma, verdana, helvetica; color: #000;">' + _('Welcome back, please wait while the credentials are finally processed.') + '</p></td></tr></table>'		    		
    	});
    	waitWindow.show();
    	grid.disable();
    	startListener2(grid);
    	dss.raiseEvent('event-twitterer',{'actions':'verifyAuthorisation','oauth_token':oauth_token,'oauth_verifier':oauth_verifier});		
     },       
    waitAndReload : function(grid,parameter) {
    	grid.setStatus(_('Saving ...'));
        var me = this;
        
    	dss.pushAndQuery('event-twitterer',parameter,
		         'event-twitterer.saved',function(eventData) {
    		 me.loadParameter(grid);
			 return true;
		});
        
    },
    
    /**
     * Updates the status text under the grid.
     * @param [in] grid
     */
    updateStatus : function(grid) {
        var newStatusText;
    	var userAccount = '';
    	var sString = dss.ajaxSyncRequest('/json/property/query', {
	        'query' : '/scripts/event-twitterer(screenName)'
	    });
	    var result = Ext.JSON.decode(sString);
	    if ((result.result!=null) && (result.result['event-twitterer']!=null)) {
    		if (result.result['event-twitterer'][0]!=null) {		
    			userAccount = result.result['event-twitterer'][0]['screenName'];
    		}
	    }
       if (('' != userAccount) && (undefined != userAccount)) { //successfully registered to twitter account
        	newStatusText = _('Event-Twitterer registered to twitter account : ') + userAccount;
        	grid.setStatus(_('Ready'), newStatusText);
        	updateTwitterAuthenticationButtons(grid, /*accountRegistered*/ true);
        } else { //not registered to twitter account
        	newStatusText = _('Authentication with Twitter account missing');
        	grid.setStatus(_('Not ready'), newStatusText);
        	updateTwitterAuthenticationButtons(grid, /*accountRegistered*/ false);
        }   	
    },
    
    /**
     * show dialog and clean up in case of failure during authentication process
     * @param grid [in]
     */
    showAuthenticationFailed : function(grid) {
	    messageBox = Ext.create('Ext.window.Window',{
			padding:"10 10 10 10",
			header:false,
			closeable:false,
			html:'<table width="100%" style="height:90px;"><tr><td><p style="text-align: center; font: 12px arial, tahoma, verdana, helvetica; color: #000;">' + _('We are sorry, the authentication process failed! <br>To retry use the "Connect to Twitter" button.')	+
			'</p></td></tr><tr><td style="text-align:center"><button onclick="hideMsgBox();">' +_('OK') + '</button></td></tr></table>'
		});
	    messageBox.show();
	    grid.disable();
	    this.deleteTwitterCredentials(grid);
    },
    
    showCredentialsOutdated : function(grid) {
	    messageBox = Ext.create('Ext.window.Window',{
			padding:"10 10 10 10",
			header:false,
			closeable:false,
			html:'<table width="100%" style="height:90px;"><tr><td><p style="text-align: center; font: 12px arial, tahoma, verdana, helvetica; color: #000;">' + _('Sorry, your Twitter credentials are outdated<br>To renew use the "Connect to Twitter" button.')	+
			'</p></td></tr><tr><td style="text-align:center"><button onclick="hideMsgBox();">' + _('OK') + '</button></td></tr></table>'
		});
	    messageBox.show();
	    grid.disable();
	    this.deleteTwitterCredentials(grid);
    },
    
    showSendingTestResult : function(succeeded) {
    	var resultTxt;
    	if (succeeded) {
    		resultTxt = _('Sending test tweet succeeded.');
	    } else {
	    	resultTxt = _('Sending test tweet failed.');
	    }
    	messageBox = Ext.create('Ext.window.Window',{
			padding:"10 10 10 10",
			header:false,
			closeable:false,
			html:'<table width="100%" style="height:90px;"><tr><td><p style="text-align: center; font: 12px arial, tahoma, verdana, helvetica; color: #000;">' + resultTxt	+
			'</p></td></tr><tr><td style="text-align:center"><button onclick="hideMsgBox();">' + _('OK') + '</button></td></tr></table>'
		});
	    messageBox.show();
	    grid.disable();
    },
    
    /**
     * Show confirmation dialog on "Diconnect from Twitter" request
     * @param grid [in]
     */
    showAreYouSure : function(grid) {
    	me = this;
	    messageBox = Ext.create('Ext.window.Window',{
			padding:"10 10 10 10",
			header:false,
			closeable:false,
			html:'<table width="100%" style="height:90px;"><tr><td><p style="text-align: center; font: 12px arial, tahoma, verdana, helvetica; color: #000;">' + _('Are you sure to disconnect from Twitter.')	+
			'</p></td></tr><tr><td style="text-align:center"><button onclick="me.deleteTwitterCredentials(grid);hideMsgBox();">' +
	     	_('Diconnect') + '</button>&nbsp; &nbsp; &nbsp;<button onclick="hideMsgBox();">' + _('cancel') + '</button></td></tr></table>'
		});
	    messageBox.show();
	    grid.disable();
    },

    /**
     * Clean up the property tree and update status bar abd buttons.
     * Used in case of failed authentication process and on "Disconnect from Twitter" button
     * @param grid [in]
     */
    deleteTwitterCredentials : function(grid) {
    	/*
    	removeProperty('/scripts/event-twitterer/requestToken');
    	removeProperty('/scripts/event-twitterer/requestTokenSecret');
    	removeProperty('/scripts/event-twitterer/accessToken');
    	removeProperty('/scripts/event-twitterer/accessTokenSecret');
    	removeProperty('/scripts/event-twitterer/userId');
    	removeProperty('/scripts/event-twitterer/screenName');  
    	this.updateStatus(grid);
    	*/
    	startCredentialsDeletedListener();
    	dss.raiseEvent('event-twitterer',{'actions':'deleteCredentials'});
    }
});

/**
 * closes the Signin Dialog in case user cancel action
 * @param grid [in]
 */
function closeSignInDialog(grid){
	waitWindow.hide();
	oEventTwitterer.updateStatus(grid);
	grid.enable();
}

/**
 * Hide "Authentification failed dialog.
 */

function hideMsgBox() 
{
    oGrid.enable();
    messageBox.hide();
    oEventTwitterer.updateStatus(oGrid);
}

/**
 * Helper to remove property from property tree
 * @param sProp [in] property path
 */
function removeProperty(sProp)
{
	var sURL="";
	var sPfad=escape(sProp);

	sURL='/json/property/remove';
	
	// '/json/property/remove?path=' + sPfad + "&value=true&flag=ARCHIVE";

	dss.ajaxAssyncRequestWithoutAnswer(sURL,{'path':sPfad});
}

/**
 * Verifies token returned in step 2 matches token we got in step 1 of authentication process.
 * @param oauth_token [in] token received in step 2
 * @param oauth_verifier [in] verifier received in step 2
 * @param propertyToken [in] token received in step 1
 * @returns {Boolean}
 */
function verifyTokenMatch(oauth_token, oauth_verifier, propertyToken) {
	if (oauth_token!='' && oauth_verifier!= '') {
    	if (propertyToken === oauth_token) {	
    		return true;
        } else {
        	//twitter send wrong request token
        	return false;
        }
	} else {
		//twitter does not return oauth_token and/or oauth_verifier at all
		return false;
	}	
}

/**
 * Update the Twitter Connect/Disconnet button visibility.
 * @param grid [in] grid buttons belong to
 * @param accountRegistered [in] status of registration
 */
function updateTwitterAuthenticationButtons(grid, accountRegistered) {

	if (true === accountRegistered) {
		grid.btnCustomRAlign[/*register*/0].setDisabled(true);
		grid.btnCustomRAlign[/*register*/0].hide();
		grid.btnCustomRAlign[/*deregister*/1].setDisabled(false);
		grid.btnCustomRAlign[/*deregister*/1].show();
	} else {
		grid.btnCustomRAlign[/*register*/0].setDisabled(false);
		grid.btnCustomRAlign[/*register*/0].show();
		grid.btnCustomRAlign[/*deregister*/1].setDisabled(true);	
		grid.btnCustomRAlign[/*deregister*/1].hide();
	}

}

function startListener() {
	var iSubscriptID = Math.floor(Math.random() * 100000000000);
	dss.ajaxSyncRequest('/json/event/subscribe', {
	    subscriptionID : iSubscriptID,
	    name : 'event-twitterer.authorisation'
	});
    dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
        subscriptionID : iSubscriptID,
        timeout : 60 * 1000
    }, function(answer) {   
        //first of all unsubscribe
        dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
            subscriptionID : iSubscriptID,
            name : 'event-twitterer.authorisation'
        });

        var result = Ext.JSON.decode(answer);
        var step1Succeeded = false;
        var sToken;
        if (result.ok==true) {
            if ((result.result!=null) && (result.result.events!=null) && (0<result.result.events.length)) {
                for (var iIndex=0;iIndex<result.result.events.length;iIndex++) {
                    if (result.result.events[iIndex].properties.step=="1") {
                        if (('true' === result.result.events[iIndex].properties.complete) &&  (result.result.events[iIndex].properties.token)) {
                            sToken=result.result.events[iIndex].properties.token;
                            step1Succeeded = true;
                        }
                    }
                }
            }       
        }
        if (step1Succeeded) {
            // Step 2 of authentication process: Redirecting the user
            var langSuffix = dss.staticDataModel.activeLanguage;
            var index = langSuffix.indexOf('_');
            langSuffix = langSuffix.slice(0, index);
            document.location.href="https://api.twitter.com/oauth/authenticate?oauth_token=" + sToken + "&force_login=true&lang="+langSuffix;            
        } else {
            // Step 1 of authentication process failed.
            waitWindow.hide();
            oEventTwitterer.showAuthenticationFailed(oGrid);                                    
        }
    });
}

function step1(){
	//hide SignInRequest dialog
    waitWindow.hide();

    waitWindow=Ext.create('Ext.window.Window',{
		height:120,
		width:320,
		padding:"10 10 10 10",
		header:false,
		closeable:false,
		html:'<table  style="height:90px;"><tr><td><p style="text-align: center; font: 12px arial, tahoma, verdana, helvetica; color: #000;">' + _('Please wait, you will be redirected to twitter to log in and approve this application.') + '</p/</td></tr></table>'		    		
	});
	waitWindow.show();
	
	startListener();
	dss.raiseEvent('event-twitterer',{'actions':'startAuthorisation','callbackURL':prepareCallbackURL()});				
}

function startListener2(grid) {
    var iSubscriptID = Math.floor(Math.random() * 100000000000);
	dss.ajaxSyncRequest('/json/event/subscribe', {
	    subscriptionID : iSubscriptID,
	    name : 'event-twitterer.authorisation'
	});
    dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
        subscriptionID : iSubscriptID,
        timeout : 60 * 1000
    }, function(answer) {   
        //first of all unsubscribe
        dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
            subscriptionID : iSubscriptID,
            name : 'event-twitterer.authorisation'
        });

        var result = Ext.JSON.decode(answer);
        var step3Succeeded = false;
        if (result.ok==true) {
            if ((result.result!=null) && (result.result.events!=null) && (0<result.result.events.length)) {
                for (var iIndex=0;iIndex<result.result.events.length;iIndex++) {
                    if (result.result.events[iIndex].properties.step=="3") {
                        if ('true' === result.result.events[iIndex].properties.complete) {
                            step3Succeeded = true;
                        }
                    }
                }
            }
        }
        if (step3Succeeded) {
            // Step 3 of authentication process: Converting the request token to an access token successfully done
            updateTwitterAuthenticationButtons(grid, /*accountRegistered*/ true);
            oEventTwitterer.updateStatus(grid);
            grid.enable();
            waitWindow.hide();
        } else {
            // Step 3 of authentication process: Converting the request token to an access token failed
            grid.enable();
            waitWindow.hide();
            oEventTwitterer.showAuthenticationFailed(grid);
        }
    });
}


function startTestResultListener() {
    var iSubscriptID = Math.floor(Math.random() * 100000000000);
	dss.ajaxSyncRequest('/json/event/subscribe', {
	    subscriptionID : iSubscriptID,
	    name : 'event-twitterer.testresult'
	});
	dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
	    subscriptionID : iSubscriptID,
	    timeout : 60 * 1000
	}, function(answer) {	
		var result = Ext.JSON.decode(answer);
        //first of all unsubscribe
        dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
            subscriptionID : iSubscriptID,
            name : 'event-twitterer.testresult'
        });
		if (result.ok==true) {
			if ((result.result!=null) && (result.result.events!=null) && (0<result.result.events.length)) {
				for (var iIndex=0;iIndex<result.result.events.length;iIndex++) {
	        			if ('true' === result.result.events[iIndex].properties.complete) {
	        				//oEventTwitterer.showSendingTestResult(true);
	        			} else {
	        				oEventTwitterer.showSendingTestResult(false);
	        			}
				}
			}		
		}
	});
}

/**
 * Listener for event-twitterer.credentialsDeleted event.
 * We have to update our status.
 */
function startCredentialsDeletedListener() {
    var iSubscriptID = Math.floor(Math.random() * 100000000000);
	dss.ajaxSyncRequest('/json/event/subscribe', {
	    subscriptionID : iSubscriptID,
	    name : 'event-twitterer.credentialsDeleted'
	});
	dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
	    subscriptionID : iSubscriptID,
	    timeout : 60 * 1000
	}, function(answer) {	
		var result = Ext.JSON.decode(answer);
		//first of all unsubscribe
        dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
            subscriptionID : iSubscriptID,
            name : 'event-twitterer.credentialsDeleted'
        });
        oEventTwitterer.updateStatus(oGrid);
	});
}

Ext.onReady(function() {
    dss.buildUpLang(['locale/{languageSuffix}/event-twitterer.po']);
    var TwitterEdit = Ext.create('DSS.addon.EventTwitterer');
    oEventTwitterer = TwitterEdit;
    TwitterEdit.initPage();
});
